Plant.data<-read.table("HW1Q3.txt")

get.outliers<-function(x,y,distance,median,cutoff){
    ## This function extracts all observations further from the median than the
    ## cut-off value, using the distance provided.
    return(list("x"=x[distance[median,]>cutoff,],"y"=y[distance[median,]>cutoff]))
}

library(glmnet) # for LASSO
X<-model.matrix(yield~.,data=Plant.data)
# This creates a matrix of predictors to be used in LASSO, converting
# categorical variables to indicators.

### Calculate a distance matrix between predictors.
inv.cov<-solve(var(X[,-1]))
for(i in seq_len(n)){
    for(j in seq_len(n)){
        diff<-X[i,-1]-X[j,-1]
        distance[i,j]<-sqrt(t(diff)%*%inv.cov%*%diff)
    }
}

# The median point is the one that minimises total distance to other
# points.
median<-which(colSums(distance)==min(colSums(distance)))

LASSO<-cv.glmnet(X,Plant.data$yield,alpha=1)
# This performs cross-validation to select the best penalty parameter lambda.

coeffs<-LASSO$glmnet.fit$beta[,LASSO$lambda.1se]
### Using one standard deviation above the smallest is common practice
### to ensure a sparse model

### Extract the outliers at 10 cut-off values
cut.offs<-6+seq_len(10)/10 
outliers<-get.outliers(X,Plant.data$yield,distance,median,cut.offs)

### For each cut-off, calculate the Mean Squared Error for the outliers
### Not including the intercept in the prediction
MSE<-rep(1,10)
for(i in seq_len(10)){
    predictions<-outliers[[i]]$x[,-1]%*%coeffs[-1]
    true<-outliers[[i]]$y
    MSE[i]<-mean((predictions-true)^2)
}
